==> 0.0.7.tar.gz: lib/widgets/spark_text_field.dart <==
part of spark_widget;

class SparkTextField extends SparkFormWidget {
  const SparkTextField({
    Key? key,
    required FormGroupItem manage,
    this.initialValue,
    this.keyboardType,
    this.textInputAction,
    this.style,
    this.strutStyle,
    this.textDirection,
    this.textAlignVertical,
    this.toolbarOptions,
    this.showCursor,
    this.smartDashesType,
    this.smartQuotesType,
    this.maxLengthEnforcement,
    this.minLines,
    this.maxLength,
    this.onChanged,
    this.onTap,
    this.onEditingComplete,
    this.onFieldSubmitted,
    this.onSaved,
    this.validator,
    this.inputFormatters,
    this.enabled,
    this.cursorHeight,
    this.cursorRadius,
    this.cursorColor,
    this.keyboardAppearance,
    this.selectionControls,
    this.buildCounter,
    this.scrollPhysics,
    this.autoFillHints,
    this.autoValidateMode,
    this.scrollController,
    this.textCapitalization,
    this.autoFocus,
    this.readOnly,
    this.obscuringCharacter,
    this.obscureText,
    this.autoCorrect,
    this.enableSuggestions,
    this.maxLines,
    this.expands,
    this.cursorWidth,
    this.scrollPadding,
    this.enableIMEPersonalizedLearning,
    this.enableInteractiveSelection,
    this.width,
    this.suffix,
    this.prefix,
    this.cleanble,
    this.cleanbleWidget,
    this.keepCleanbleVisible,
    this.cleanbleSize,
    this.contentPadding,
    this.constraints,
    this.hitText,
    this.hitTextStyle,
    this.counterText,
    this.showConuter,
    this.counterStyle,
  }) : super(key: key, manage: manage);

  final TextStyle? counterStyle;
  final String? counterText;
  final bool? showConuter;
  final String? initialValue;
  final TextInputType? keyboardType;
  final TextCapitalization? textCapitalization;
  final TextInputAction? textInputAction;
  final TextStyle? style;
  final StrutStyle? strutStyle;
  final TextDirection? textDirection;
  final TextAlign textAlign = TextAlign.start;
  final TextAlignVertical? textAlignVertical;
  final bool? autoFocus;
  final bool? readOnly;
  final ToolbarOptions? toolbarOptions;
  final bool? showCursor;
  final String? obscuringCharacter;
  final bool? obscureText;
  final bool? autoCorrect;
  final SmartDashesType? smartDashesType;
  final SmartQuotesType? smartQuotesType;
  final bool? enableSuggestions;
  final MaxLengthEnforcement? maxLengthEnforcement;
  final int? maxLines;
  final int? minLines;
  final bool? expands;
  final int? maxLength;
  final ValueChanged<String>? onChanged;
  final GestureTapCallback? onTap;
  final VoidCallback? onEditingComplete;
  final ValueChanged<String>? onFieldSubmitted;
  final FormFieldSetter<String>? onSaved;
  final FormFieldValidator<String>? validator;
  final List<TextInputFormatter>? inputFormatters;
  final bool? enabled;
  final double? cursorWidth;
  final double? cursorHeight;
  final Radius? cursorRadius;
  final Color? cursorColor;
  final Brightness? keyboardAppearance;
  final EdgeInsets? scrollPadding;
  final TextSelectionControls? selectionControls;
  final InputCounterWidgetBuilder? buildCounter;
  final ScrollPhysics? scrollPhysics;
  final Iterable<String>? autoFillHints;
  final AutovalidateMode? autoValidateMode;
  final ScrollController? scrollController;
  final bool? enableIMEPersonalizedLearning;
  final bool? enableInteractiveSelection;
  final double? width;
  final Widget? suffix;
  final Widget? prefix;
  final bool? cleanble;
  final Widget? cleanbleWidget;
  final Size? cleanbleSize;
  final bool? keepCleanbleVisible;
  final EdgeInsets? contentPadding;
  final BoxConstraints? constraints;
  final String? hitText;
  final TextStyle? hitTextStyle;

  @override
  State<StatefulWidget> createState() => _SparkTextFieldBase();
}

class _SparkTextFieldBase extends SparkFormWidgetBase<SparkTextField, String> {
  SparkTheme? theme;

  @override
  void initState() {
    _controller.text = fieldData;
    super.initState();
  }

  @override
  void valueChangeNotify() {
    _controller.text = fieldData;
    _controller.selection = TextSelection.fromPosition(
      TextPosition(
        affinity: TextAffinity.upstream,
        offset: fieldData.length,
      ),
    );
  }

  void _handleReset() {
    widget.manage.updateValue('', true);
    _handleChange('');
  }

  final TextEditingController _controller = TextEditingController();

  @override
  void dispose() {
    _controller.dispose();
    super.dispose();
  }

  /// handle textField input change
  /// notify parent form item handle value change
  /// see [SparkFormItem]
  void _handleChange(String value) {
    widget.manage.updateValue(value, false);
    handleFormDataChange(value);
  }

  /// build default counter
  Widget? _buildDefaultCounter(
    BuildContext context, {
    required int currentLength,
    required int? maxLength,
    required bool isFocused,
  }) {
    String _maxCount = maxLength != null ? '/$maxLength' : '';
    TextStyle _counterStyle = widget.counterStyle ??
        TextStyle(
          fontSize: theme?.hitFontSize,
          color: theme?.hitColor,
        );
    return Text(
      widget.counterText ?? '$currentLength$_maxCount',
      style: _counterStyle,
    );
  }

  @override
  Widget build(BuildContext context) {
    theme = SparkThemeData.of(context);
    bool obscureText = widget.obscureText ?? false;
    bool _enable = widget.enabled ?? true;
    bool _showSuffix = widget.suffix != null;
    bool _needShowCleanble = widget.cleanble ?? false;
    Size _cleanbleSize = widget.cleanbleSize ?? const Size(18, 18);
    InputCounterWidgetBuilder? _buildCounter = (widget.showConuter ?? false)
        ? (widget.buildCounter ?? _buildDefaultCounter)
        : null;
    return Row(
      crossAxisAlignment: CrossAxisAlignment.center,
      children: [
        if (widget.prefix != null) widget.prefix!,
        Expanded(
            flex: 1,
            child: SizedBox(
              width: widget.width ?? double.infinity,
              child: TextField(
                controller: _controller,
                focusNode: widget.manage.focusNode,
                decoration: InputDecoration(
                  isCollapsed: true,
                  filled: false,
                  contentPadding: EdgeInsets.zero,
                  hintText: widget.hitText,
                  hintStyle: widget.hitTextStyle,
                  // counter: widget.conuter,
                  // counterText: widget.counterText,
                  // counterStyle: widget.counterStyle,
                  hintTextDirection: widget.textDirection,
                  border:
                      const UnderlineInputBorder(borderSide: BorderSide.none),
                ),
                keyboardType: widget.keyboardType,
                textInputAction: widget.textInputAction,
                style: widget.style ??
                    TextStyle(
                      color: _enable
                          ? theme?.formItemColor
                          : theme?.formItemDisabledColor,
                      fontSize: theme?.formItemFontSize,
                    ),
                strutStyle: widget.strutStyle,
                textAlign: widget.textAlign,
                textAlignVertical: widget.textAlignVertical,
                textDirection: widget.textDirection,
                textCapitalization:
                    widget.textCapitalization ?? TextCapitalization.none,
                autofocus: widget.autoFocus ?? false,
                toolbarOptions: widget.toolbarOptions,
                readOnly: widget.readOnly ?? false,
                showCursor: widget.showCursor,
                obscuringCharacter: widget.obscuringCharacter ?? '•',
                obscureText: obscureText,
                autocorrect: widget.autoCorrect ?? true,
                smartDashesType: widget.smartDashesType ??
                    (obscureText
                        ? SmartDashesType.disabled
                        : SmartDashesType.enabled),
                smartQuotesType: widget.smartQuotesType ??
                    (obscureText
                        ? SmartQuotesType.disabled
                        : SmartQuotesType.enabled),
                enableSuggestions: widget.enableSuggestions ?? true,
                maxLengthEnforcement: widget.maxLengthEnforcement,
                maxLines: widget.maxLines,
                minLines: widget.minLines,
                expands: widget.expands ?? false,
                maxLength: widget.maxLength,
                onChanged: _handleChange,
                onTap: widget.onTap,
                onEditingComplete: widget.onEditingComplete,
                onSubmitted: widget.onFieldSubmitted,
                inputFormatters: widget.inputFormatters,
                enabled: _enable,
                cursorWidth: widget.cursorWidth ?? 2.0,
                cursorHeight: widget.cursorHeight,
                cursorRadius: widget.cursorRadius,
                cursorColor: widget.cursorColor,
                scrollPadding:
                    widget.scrollPadding ?? const EdgeInsets.all(20.0),
                scrollPhysics: widget.scrollPhysics,
                keyboardAppearance: widget.keyboardAppearance,
                enableInteractiveSelection:
                    widget.enableInteractiveSelection ?? true,
                selectionControls: widget.selectionControls,
                buildCounter: _buildCounter,
                autofillHints: widget.autoFillHints,
                scrollController: widget.scrollController,
                enableIMEPersonalizedLearning:
                    widget.enableIMEPersonalizedLearning ?? true,
              ),
            )),
        if (_showSuffix) widget.suffix!,
        if (_needShowCleanble)
          SparkWrapper<String>(
            builder: (_, _value, _u) => SizedBox(
              width: _cleanbleSize.width,
              height: _cleanbleSize.height,
              child: _value.isNotEmpty ||
                      _value.isEmpty && (widget.keepCleanbleVisible ?? false)
                  ? GestureDetector(
                      behavior: HitTestBehavior.opaque,
                      onTap: () {
                        _handleReset();
                      },
                      child: widget.cleanbleWidget ??
                          UnconstrainedBox(
                            child: Image.asset(
                              'icons/icon_delete.png',
                              width: 14,
                              height: 14,
                              package: 'spark_form',
                            ),
                          ),
                    )
                  : null,
            ),
            manage: widget.manage,
          ),
      ],
    );
  }

  @override
  bool handleGetValueChangeNotification(
    SparkBeaconNotification<String> notification,
  ) {
    // no out side change function
    return true;
  }
}

class SparkInputDecoration {
  String? hitText;
  TextStyle? hitTextStyle;
  Widget? counter;
  String? counterText;
}

==> 0.0.8.tar.gz: lib/widgets/spark_text_field.dart <==
part of spark_widget;

class SparkTextField extends SparkFormWidget {
  const SparkTextField({
    Key? key,
    required FormGroupItem manage,
    this.initialValue,
    this.keyboardType,
    this.textInputAction,
    this.style,
    this.strutStyle,
    this.textDirection,
    this.textAlignVertical,
    this.toolbarOptions,
    this.showCursor,
    this.smartDashesType,
    this.smartQuotesType,
    this.maxLengthEnforcement,
    this.minLines,
    this.maxLength,
    this.onChanged,
    this.onTap,
    this.onEditingComplete,
    this.onFieldSubmitted,
    this.onSaved,
    this.validator,
    this.inputFormatters,
    this.cursorHeight,
    this.cursorRadius,
    this.cursorColor,
    this.keyboardAppearance,
    this.selectionControls,
    this.buildCounter,
    this.scrollPhysics,
    this.autoFillHints,
    this.autoValidateMode,
    this.scrollController,
    this.textCapitalization,
    this.autoFocus,
    this.readOnly,
    this.obscuringCharacter,
    this.obscureText,
    this.autoCorrect,
    this.enableSuggestions,
    this.maxLines,
    this.expands,
    this.cursorWidth,
    this.scrollPadding,
    this.enableIMEPersonalizedLearning,
    this.enableInteractiveSelection,
    this.width,
    this.suffix,
    this.prefix,
    this.clearable,
    this.clearableWidget,
    this.keepClearableVisible,
    this.clearableSize,
    this.contentPadding,
    this.constraints,
    this.hitText,
    this.hitTextStyle,
    this.counterText,
    this.showCounter,
    this.counterStyle,
    bool enabled = true,
  }) : super(key: key, manage: manage, enabled:  enabled);

  final TextStyle? counterStyle;
  final String? counterText;
  final bool? showCounter;
  final String? initialValue;
  final TextInputType? keyboardType;
  final TextCapitalization? textCapitalization;
  final TextInputAction? textInputAction;
  final TextStyle? style;
  final StrutStyle? strutStyle;
  final TextDirection? textDirection;
  final TextAlign textAlign = TextAlign.start;
  final TextAlignVertical? textAlignVertical;
  final bool? autoFocus;
  final bool? readOnly;
  final ToolbarOptions? toolbarOptions;
  final bool? showCursor;
  final String? obscuringCharacter;
  final bool? obscureText;
  final bool? autoCorrect;
  final SmartDashesType? smartDashesType;
  final SmartQuotesType? smartQuotesType;
  final bool? enableSuggestions;
  final MaxLengthEnforcement? maxLengthEnforcement;
  final int? maxLines;
  final int? minLines;
  final bool? expands;
  final int? maxLength;
  final ValueChanged<String>? onChanged;
  final GestureTapCallback? onTap;
  final VoidCallback? onEditingComplete;
  final ValueChanged<String>? onFieldSubmitted;
  final FormFieldSetter<String>? onSaved;
  final FormFieldValidator<String>? validator;
  final List<TextInputFormatter>? inputFormatters;
  final double? cursorWidth;
  final double? cursorHeight;
  final Radius? cursorRadius;
  final Color? cursorColor;
  final Brightness? keyboardAppearance;
  final EdgeInsets? scrollPadding;
  final TextSelectionControls? selectionControls;
  final InputCounterWidgetBuilder? buildCounter;
  final ScrollPhysics? scrollPhysics;
  final Iterable<String>? autoFillHints;
  final AutovalidateMode? autoValidateMode;
  final ScrollController? scrollController;
  final bool? enableIMEPersonalizedLearning;
  final bool? enableInteractiveSelection;
  final double? width;
  final Widget? suffix;
  final Widget? prefix;
  final bool? clearable;
  final Widget? clearableWidget;
  final Size? clearableSize;
  final bool? keepClearableVisible;
  final EdgeInsets? contentPadding;
  final BoxConstraints? constraints;
  final String? hitText;
  final TextStyle? hitTextStyle;

  @override
  State<StatefulWidget> createState() => _SparkTextFieldBase();
}

class _SparkTextFieldBase extends SparkFormWidgetBase<SparkTextField, String> {
  SparkTheme? theme;

  @override
  void initState() {
    _controller.text = fieldData;
    super.initState();
  }

  @override
  void valueChangeNotify() {
    _controller.text = fieldData;
    _controller.selection = TextSelection.fromPosition(
      TextPosition(
        affinity: TextAffinity.upstream,
        offset: fieldData.length,
      ),
    );
  }

  void _handleReset() {
    widget.manage.updateValue('', true);
    _handleChange('');
  }

  final TextEditingController _controller = TextEditingController();

  @override
  void dispose() {
    _controller.dispose();
    super.dispose();
  }

  /// handle textField input change
  /// notify parent form item handle value change
  /// see [SparkFormItem]
  void _handleChange(String value) {
    widget.manage.updateValue(value, false);
    handleFormDataChange(value);
  }

  /// build default counter
  Widget? _buildDefaultCounter(
    BuildContext context, {
    required int currentLength,
    required int? maxLength,
    required bool isFocused,
  }) {
    String _maxCount = maxLength != null ? '/$maxLength' : '';
    TextStyle _counterStyle = widget.counterStyle ??
        TextStyle(
          fontSize: theme?.hitFontSize,
          color: theme?.hitColor,
        );
    return Text(
      widget.counterText ?? '$currentLength$_maxCount',
      style: _counterStyle,
    );
  }

  @override
  Widget build(BuildContext context) {
    theme = SparkThemeData.of(context);
    bool obscureText = widget.obscureText ?? false;
    bool _enable = widget.enabled;
    bool _showSuffix = widget.suffix != null;
    bool _needShowClearable = widget.clearable ?? false;
    Size _clearableSize = widget.clearableSize ?? const Size(18, 18);
    InputCounterWidgetBuilder? _buildCounter = (widget.showCounter ?? false)
        ? (widget.buildCounter ?? _buildDefaultCounter)
        : null;
    return Row(
      crossAxisAlignment: CrossAxisAlignment.center,
      children: [
        if (widget.prefix != null) widget.prefix!,
        Expanded(
            flex: 1,
            child: SizedBox(
              width: widget.width ?? double.infinity,
              child: TextField(
                controller: _controller,
                focusNode: widget.manage.focusNode,
                decoration: InputDecoration(
                  isCollapsed: true,
                  filled: false,
                  contentPadding: EdgeInsets.zero,
                  hintText: widget.hitText,
                  hintStyle: widget.hitTextStyle,
                  hintTextDirection: widget.textDirection,
                  border:
                      const UnderlineInputBorder(borderSide: BorderSide.none),
                ),
                keyboardType: widget.keyboardType,
                textInputAction: widget.textInputAction,
                style: widget.style ??
                    TextStyle(
                      color: _enable
                          ? theme?.formItemColor
                          : theme?.formItemDisabledColor,
                      fontSize: theme?.formItemFontSize,
                    ),
                strutStyle: widget.strutStyle,
                textAlign: widget.textAlign,
                textAlignVertical: widget.textAlignVertical,
                textDirection: widget.textDirection,
                textCapitalization:
                    widget.textCapitalization ?? TextCapitalization.none,
                autofocus: widget.autoFocus ?? false,
                toolbarOptions: widget.toolbarOptions,
                readOnly: widget.readOnly ?? false,
                showCursor: widget.showCursor,
                obscuringCharacter: widget.obscuringCharacter ?? '•',
                obscureText: obscureText,
                autocorrect: widget.autoCorrect ?? true,
                smartDashesType: widget.smartDashesType ??
                    (obscureText
                        ? SmartDashesType.disabled
                        : SmartDashesType.enabled),
                smartQuotesType: widget.smartQuotesType ??
                    (obscureText
                        ? SmartQuotesType.disabled
                        : SmartQuotesType.enabled),
                enableSuggestions: widget.enableSuggestions ?? true,
                maxLengthEnforcement: widget.maxLengthEnforcement,
                maxLines: widget.maxLines,
                minLines: widget.minLines,
                expands: widget.expands ?? false,
                maxLength: widget.maxLength,
                onChanged: _handleChange,
                onTap: widget.onTap,
                onEditingComplete: widget.onEditingComplete,
                onSubmitted: widget.onFieldSubmitted,
                inputFormatters: widget.inputFormatters,
                enabled: _enable,
                cursorWidth: widget.cursorWidth ?? 2.0,
                cursorHeight: widget.cursorHeight,
                cursorRadius: widget.cursorRadius,
                cursorColor: widget.cursorColor,
                scrollPadding:
                    widget.scrollPadding ?? const EdgeInsets.all(20.0),
                scrollPhysics: widget.scrollPhysics,
                keyboardAppearance: widget.keyboardAppearance,
                enableInteractiveSelection:
                    widget.enableInteractiveSelection ?? true,
                selectionControls: widget.selectionControls,
                buildCounter: _buildCounter,
                autofillHints: widget.autoFillHints,
                scrollController: widget.scrollController,
                enableIMEPersonalizedLearning:
                    widget.enableIMEPersonalizedLearning ?? true,
              ),
            )),
        if (_showSuffix) widget.suffix!,
        if (_needShowClearable)
          SparkWrapper<String>(
            builder: (_, _value, _u) => SizedBox(
              width: _clearableSize.width,
              height: _clearableSize.height,
              child: _value.isNotEmpty ||
                      _value.isEmpty && (widget.keepClearableVisible ?? false)
                  ? GestureDetector(
                      behavior: HitTestBehavior.opaque,
                      onTap: () {
                        _handleReset();
                      },
                      child: widget.clearableWidget ??
                          UnconstrainedBox(
                            child: Image.asset(
                              'icons/icon_delete.png',
                              width: 14,
                              height: 14,
                              package: 'spark_form',
                            ),
                          ),
                    )
                  : null,
            ),
            manage: widget.manage,
          ),
      ],
    );
  }

  @override
  bool handleGetValueChangeNotification(
    SparkBeaconNotification<String> notification,
  ) {
    // it's not work
    _handleChange(notification.value);
    return true;
  }
}

class SparkInputDecoration {
  String? hitText;
  TextStyle? hitTextStyle;
  Widget? counter;
  String? counterText;
}
